<?php
/**
 * Functions for compatability with WP-Rocket
 * Plugin: https://wp-rocket.me/ .
 *
 * @package PeachPay
 */

if ( ! defined( 'PEACHPAY_ABSPATH' ) ) {
	exit;
}

add_action( 'peachpay_post_plugin_update_actions', 'peachpay_clear_rocket_cache' );
add_action( 'peachpay_init_compatibility', 'rocket_compatability_init' );

/**
 * Clear everything in the wp cache cache so we can update our js files.
 */
function peachpay_clear_rocket_cache() {
	if ( function_exists( 'rocket_clean_domain' ) ) {
		rocket_clean_domain();
	}
}

/**
 * We run into issues when our files are minify'd and defered so just exclude us if we aren't already.
 */
function rocket_compatability_init() {
	if ( peachpay_get_settings_option( 'peachpay_currency_options', 'enabled' ) ) {
		add_filter( 'do_rocket_generate_caching_files', '__return_false', 10000000 );
		peachpay_clear_rocket_cache();
	}
	$settings = get_option( 'wp_rocket_settings' );
	$excluded = $settings['exclude_js'];
	$deferred = $settings['exclude_defer_js'];
	$delayed  = $settings['delay_js_exclusions'];

	if ( ! empty( $excluded ) && ! in_array( '/wp-content/plugins/peachpay-for-woocommerce', $excluded, true ) ) {
		array_push( $excluded, '/wp-content/plugins/peachpay-for-woocommerce' );
		$settings['exclude_js'] = $excluded;
	}
	if ( ! empty( $deferred ) && ! in_array( '/wp-content/plugins/peachpay-for-woocommerce', $deferred, true ) ) {
		array_push( $deferred, '/wp-content/plugins/peachpay-for-woocommerce' );
		$settings['exclude_defer_js'] = $deferred;
	}
	if ( ! empty( $delayed ) && ! in_array( '/wp-content/plugins/peachpay-for-woocommerce', $delayed, true ) ) {
		array_push( $delayed, '/wp-content/plugins/peachpay-for-woocommerce' );
		$settings['delay_js_exclusions'] = $delayed;
	}
	update_option( 'wp_rocket_settings', $settings );
}
