<?php
/**
 * PeachPay payment settings.
 *
 * @package PeachPay
 */

if ( ! defined( 'PEACHPAY_ABSPATH' ) ) {
	exit;
}

/**
 * Registers each payment option setting.
 */
function peachpay_settings_payment() {

	if ( peachpay_approved_wc_api_access() ) {
		update_option( 'peachpay_valid_key', true );
	}

	if ( ! get_option( 'peachpay_valid_key' ) && ! peachpay_is_test_mode() ) {
		add_settings_section(
			'peachpay_section_payment_cannot_continue',
			__( 'Payment methods', 'peachpay-for-woocommerce' ),
			'peachpay_section_payment_cannot_continue_html',
			'peachpay'
		);
		return;
	}

	add_settings_section(
		'peachpay_section_payment',
		'',
		'peachpay_feedback_cb',
		'peachpay'
	);

	add_settings_field(
		'peachpay_payment_general_setting',
		__( 'General', 'peachpay-for-woocommerce' ),
		'peachpay_general_setting_section',
		'peachpay',
		'peachpay_section_payment',
		array( 'class' => 'pp-header' )
	);

	add_settings_field(
		'peachpay_paypal_setting',
		__( 'PayPal', 'peachpay-for-woocommerce' ),
		'peachpay_paypal_setting_section',
		'peachpay',
		'peachpay_section_payment',
		array( 'class' => 'pp-header' )
	);

	add_settings_field(
		'peachpay_stripe_setting',
		__( 'Stripe', 'peachpay-for-woocommerce' ),
		'peachpay_stripe_setting_section',
		'peachpay',
		'peachpay_section_payment',
		array( 'class' => 'pp-header' )
	);

	add_settings_field(
		'peachpay_purchase_order_setting',
		__( 'Purchase order', 'peachpay-for-woocommerce' ),
		'peachpay_purchase_order_setting_section',
		'peachpay',
		'peachpay_section_payment',
		array( 'class' => 'pp-header' )
	);

	add_settings_field(
		'peachpay_cod_check_bacs_setting',
		__( 'WooCommerce methods', 'peachpay-for-woocommerce' ),
		'peachpay_cod_check_bacs_setting_section',
		'peachpay',
		'peachpay_section_payment',
		array( 'class' => 'pp-header' )
	);
}

/**
 * Extra settings for the Stripe card_payments method.
 */
function peachpay_stripe_card_payments_extra_settings() {
	?>
	<br>
	<?php

	peachpay_admin_input(
		'stripe_card_force_3d_secure',
		'peachpay_payment_options',
		'stripe_card_force_3d_secure',
		1,
		__( 'Force 3D Secure', 'peachpay-for-woocommerce' ),
		__( 'Stripe normally determines when 3D Secure should be shown automatically. If "Force 3D Secure" is enabled, 3D Secure will be shown for all card transactions. In test mode, 3D Secure is only shown for 3DS test cards regardless of the setting.', 'peachpay-for-woocommerce' ),
		array(
			'input_type' => 'checkbox',
		)
	);

	?>
	<br/>
	<?php

	peachpay_admin_input(
		'stripe_card_capture_method',
		'peachpay_payment_options',
		'stripe_card_capture_method',
		'automatic',
		__( 'Charge type', 'peachpay-for-woocommerce' ),
		__( 'This option determines if the customers funds are captured immediately or only authorized for capture at a later time. Authorized payments expire and cannot be captured after 7 days.', 'peachpay-for-woocommerce' ),
		array(
			'input_type'     => 'select',
			'select_options' => array(
				'automatic' => __( 'Capture', 'peachpay-for-woocommerce' ),
				'manual'    => __( 'Authorize', 'peachpay-for-woocommerce' ),
			),
		)
	);

}

/**
 * Renders the payment test mode setting section.
 */
function peachpay_general_setting_section() {
	?>
	<div class="peachpay-setting-section">
		<?php
		// PeachPay payment test mode setting.
		peachpay_field_test_mode_cb();

		// Refund on cancel.
		peachpay_admin_input(
			'dispute-cancel-order',
			'peachpay_payment_options',
			'refund_on_cancel',
			1,
			__( 'Automatically refund payment after changing order status to "cancelled" ', 'peachpay-for-woocommerce' ),
			'',
			array(
				'input_type' => 'checkbox',
			)
		);

		// Hide native checkout option.
		peachpay_admin_input(
			'peachpay-only-checkout',
			'peachpay_payment_options',
			'make_pp_the_only_checkout',
			1,
			__( 'Make PeachPay the only checkout method', 'peachpay-for-woocommerce' ),
			__( 'Hide WooCommerce native checkout buttons (not available in test mode)', 'peachpay-for-woocommerce' ),
			array(
				'input_type' => 'checkbox',
				'disabled'   => peachpay_is_test_mode(),
			)
		);
		?>
		<div class="pp-save-button-section">
			<?php peachpay_build_video_help_section( 'https://youtu.be/Nc1IYyFFOVg' ); ?>
			<?php submit_button( 'Save changes', 'pp-button-primary' ); ?>
		</div>
	</div>
	<?php
}

/**
 * Renders the paypal setting section.
 */
function peachpay_paypal_setting_section() {
	?>
	<div class="peachpay-setting-section">
		<?php
			peachpay_field_paypal_cb();

			peachpay_admin_input(
				'paypal-enable',
				'peachpay_payment_options',
				'paypal',
				1,
				__( 'Show PayPal in the checkout window', 'peachpay-for-woocommerce' ),
				'',
				array(
					'input_type' => 'checkbox',
					'disabled'   => ! peachpay_is_test_mode() && ! get_option( 'peachpay_paypal_signup' ),
				)
			);
		?>
		<div>
		<?php
		do_action( 'peachpay_paypal_extra_settings' );
		?>
		</div>
		<div class="pp-save-button-section">
			<?php peachpay_build_video_help_section( 'https://youtu.be/3yHFi0A3Jw8' ); ?>
			<?php submit_button( 'Save changes', 'pp-button-primary' ); ?>
		</div>
	</div>
	<?php
}

/**
 * Renders the stripe setting section.
 */
function peachpay_stripe_setting_section() {
	add_action( 'peachpay_action_stripe_card_payments_extra_settings', 'peachpay_stripe_card_payments_extra_settings' );

	$peachpay_connected_stripe_account = get_option( 'peachpay_connected_stripe_account', array( 'id' => '' ) );

	$connect_id = is_array( $peachpay_connected_stripe_account ) ? $peachpay_connected_stripe_account['id'] : '';
	// Capabilities are fetched on the PHP side so the merchant does not see any changes briefly after page load.
	$capabilities = peachpay_fetch_stripe_capabilities( $connect_id );
	?>
	<div class="peachpay-setting-section">
		<?php
			// Stripe connect option.
			peachpay_field_stripe_cb();

			// Stripe PeachPay checkout enable.
			peachpay_admin_input(
				'stripe-enable',
				'peachpay_payment_options',
				'enable_stripe',
				1,
				__( 'Show Stripe in the checkout window', 'peachpay-for-woocommerce' ),
				'',
				array(
					'input_type' => 'checkbox',
					'disabled'   => ! peachpay_is_test_mode() && ! get_option( 'peachpay_connected_stripe_account' ),
				)
			);

			// Cancel on dispute.
			peachpay_admin_input(
				'dispute-cancel-order',
				'peachpay_payment_options',
				'cancel_order_on_dispute',
				1,
				__( 'Change order status to "cancelled" on dispute', 'peachpay-for-woocommerce' ),
				'',
				array(
					'input_type' => 'checkbox',
				)
			);

			// Card Payment method.
			peachpay_stripe_payment_option(
				'card_payments',
				__( 'Credit card / Debit card', 'peachpay-for-woocommerce' ),
				array(
					peachpay_url( 'public/img/marks/mini-visa.svg' ),
					peachpay_url( 'public/img/marks/mini-master.svg' ),
					peachpay_url( 'public/img/marks/mini-amex.svg' ),
					peachpay_url( 'public/img/marks/mini-diners.svg' ),
				),
				__( 'Credit and debit cards (Visa, Mastercard, American Express, Discover and Diners, China UnionPay, JCB, Cartes Bancaires, Interac) are a dominant payment method globally, accounting for 41% of online payments.', 'peachpay-for-woocommerce' ),
				'https://stripe.com/pricing/local-payment-methods',
				array(
					'class'                       => 'pp-card-top',
					'no-input'                    => true,
					'data-stripe-capability-type' => 'card_payments',
					'data-stripe-capability'      => peachpay_stripe_capability( $capabilities, 'card_payments' ),
				)
			);

			// Apple Pay/ Google Pay payment method.
			peachpay_stripe_payment_option(
				'stripe_payment_request',
				'Apple Pay / Google Pay',
				peachpay_url( 'public/img/marks/apple-google.svg' ),
				__( 'Apple Pay is a wallet that allows customers to pay using payment details stored on their iPhone, iPad, or Apple Watch. Google Pay allows customers to pay with any payment method saved to their Google account.', 'peachpay-for-woocommerce' ),
				'https://stripe.com/pricing/local-payment-methods',
				array(
					'data-stripe-capability-type'     => 'card_payments',
					'data-stripe-capability'          => peachpay_stripe_capability( $capabilities, 'card_payments' ),
					'data-stripe-applepay-configured' => peachpay_get_settings_option( 'peachpay_apple_pay_settings_v2', 'apple_pay_domain_set', 'no' ) === 'yes',
				)
			);

			// Afiirm payment method.
			peachpay_stripe_payment_option(
				'affirm_payments',
				'Affirm',
				peachpay_url( 'public/img/marks/affirm.svg' ),
				__( 'Affirm has a network of more than six million shoppers in the US. Customers can choose from one of two options for payment: Split Pay and Installments. Businesses offering Affirm receive the full payment upfront and are protected against fraud and customer payment risk.', 'peachpay-for-woocommerce' ),
				'https://stripe.com/pricing/local-payment-methods',
				array(
					'data-stripe-capability-type' => 'affirm_payments',
					'data-stripe-capability'      => peachpay_stripe_capability( $capabilities, 'affirm_payments' ),
				)
			);

			// Klarna payment method.
			peachpay_stripe_payment_option(
				'klarna_payments',
				'Klarna',
				peachpay_url( 'public/img/marks/klarna.svg' ),
				__( 'Klarna offers flexible payment options that give customers more freedom to choose when and how to pay for a purchase. Klarna provides payment solutions for 90 million consumers and over 200,000 businesses across 19 markets.', 'peachpay-for-woocommerce' ),
				'https://stripe.com/pricing/local-payment-methods',
				array(
					'data-stripe-capability-type' => 'klarna_payments',
					'data-stripe-capability'      => peachpay_stripe_capability( $capabilities, 'klarna_payments' ),
				)
			);

			// Afterpay / Clearpay payment method.
			peachpay_stripe_payment_option(
				'afterpay_clearpay_payments',
				'Afterpay / Clearpay',
				peachpay_url( 'public/img/marks/afterpay.svg' ),
				__( 'Afterpay (also known as Clearpay in the UK) offers customers more payment flexibility with no credit checks, no upfront fees, and no interest for on-time payments. Afterpay has more than 13M global customers and works with 75,000 brands and retailers.', 'peachpay-for-woocommerce' ),
				'https://stripe.com/pricing/local-payment-methods',
				array(
					'data-stripe-capability-type' => 'afterpay_clearpay_payments',
					'data-stripe-capability'      => peachpay_stripe_capability( $capabilities, 'afterpay_clearpay_payments' ),
				)
			);

			peachpay_field_stripe_more_info_cb();
		?>
		<div class="pp-save-button-section">
			<?php peachpay_build_video_help_section( 'https://youtu.be/SrTykTIzwHo' ); ?>
			<?php submit_button( 'Save changes', 'pp-button-primary' ); ?>
		</div>
	</div>
	<?php
}

/**
 * Renders the Purchase Order settings section.
 *
 * @return void
 */
function peachpay_purchase_order_setting_section() {
	?>
	<div class="peachpay-setting-section">
		<?php
		peachpay_admin_input(
			'purchase-order-enable',
			'peachpay_payment_options',
			'enable_purchase_order',
			1,
			__( 'Show the Purchase Order payment method in the checkout window', 'peachpay-for-woocommerce' ),
			'',
			array(
				'input_type' => 'checkbox',
				'disabled'   => false,
			)
		);

		peachpay_admin_input(
			'peachpay-purchase-order-field-name',
			'peachpay_payment_options',
			'purchase_order_field_name',
			__( 'Purchase order', 'peachpay-for-woocommerce' ),
			__( 'Field name', 'peachpay-for-woocommerce' ),
			__( 'Customize the field name for Purchase Order. Leaving it blank defaults it to "Purchase Order" in your chosen language.', 'peachpay-for-woocommerce' ),
			array(
				'input_type' => 'text',
				'disabled'   => false,
			)
		);

		peachpay_admin_input(
			'peachpay-purchase-order-description',
			'peachpay_payment_options',
			'purchase_order_description',
			'',
			__( 'Description', 'peachpay-for-woocommerce' ),
			__( 'Customize a description to be displayed under the Purchase Order payment option. This will be left blank by default.', 'peachpay-for-woocommerce' ),
			array(
				'input_type' => 'textarea',
				'disabled'   => false,
			)
		);
		?>
		<div class="pp-save-button-section">
			<?php peachpay_build_video_help_section( 'https://youtu.be/aNi4utI2cO0' ); ?>
			<?php submit_button( 'Save changes', 'pp-button-primary' ); ?>
		</div>
	</div>
	<?php
}

/**
 * Renders the Cod, Check, and Bacs settings section.
 *
 * @return void
 */
function peachpay_cod_check_bacs_setting_section() {
	?>
	<div class="peachpay-setting-section">
		<p><?php esc_html_e( 'Cash on Delivery, Check, and Bank Transfer payment options will be available through PeachPay if they are turned on in', 'peachpay_for_woocommerce' ); ?>
			<a href="<?php /* phpcs:ignore */ echo admin_url('admin.php?page=wc-settings&tab=checkout' ); ?>"><?php esc_html_e( 'WooCommerce', 'peachpay_for_woocommerce' ); ?></a>.
		</p>
	</div>
	<?php
}

/**
 * Renders the test mode option.
 */
function peachpay_field_test_mode_cb() {
	if ( peachpay_is_test_mode() && ! peachpay_get_settings_option( 'peachpay_payment_options', 'known_testmode' ) ) {
		peachpay_set_settings_option( 'peachpay_payment_options', 'known_testmode', 1 );
	} elseif ( ! peachpay_is_test_mode() && peachpay_get_settings_option( 'peachpay_payment_options', 'known_testmode' ) ) {
		peachpay_set_settings_option( 'peachpay_payment_options', 'known_testmode', 0 );
	}

	if ( ! peachpay_paypal_enabled() ) {
		peachpay_set_settings_option( 'peachpay_payment_options', 'paypal', 0 );
	}

	?>
	<div class="pp-switch-section">
		<div>
			<label class="pp-switch">
				<input
					id="peachpay_test_mode"
					name="peachpay_payment_options[test_mode]"
					type="checkbox"
					value="1"
					<?php checked( 1, peachpay_get_settings_option( 'peachpay_payment_options', 'test_mode' ), true ); ?>
				>
				<span class="pp-slider round"></span>
			</label>
		</div>
		<div>
			<label class="pp-setting-label" for="peachpay_test_mode"><?php esc_html_e( 'Enable test mode', 'peachpay-for-woocommerce' ); ?></label>
			<p class="description">
				<?php esc_html_e( 'Make test payments with or without a connected payment method.', 'peachpay-for-woocommerce' ); ?>
			</p>
			<p class="description">
				<?php esc_html_e( 'For Stripe, use card number', 'peachpay-for-woocommerce' ); ?>&nbsp<b>4242 4242 4242 4242</b>, <?php esc_html_e( 'with expiration', 'peachpay-for-woocommerce' ); ?>&nbsp<b>04/24</b> <?php esc_html_e( 'and CVC', 'peachpay-for-woocommerce' ); ?>&nbsp<b>444</b>. <?php esc_html_e( 'For PayPal, see', 'peachpay-for-woocommerce' ); ?>&nbsp<a target="_blank" href="https://docs.peachpay.app/paypal#test-mode"><?php esc_html_e( 'these instructions', 'peachpay-for-woocommerce' ); ?></a>.
			</p>
		</div>
	</div>
	<?php
}

/**
 * Renders the please give peachpay permission notice.
 */
function peachpay_section_payment_cannot_continue_html() {
	$retry_url = get_site_url() . '/wp-admin/admin.php?page=peachpay&retry_permission=1';
	?>
	<p><?php esc_html_e( 'To continue setting up PeachPay, please', 'peachpay-for-woocommerce' ); ?>&nbsp<a href="<?php echo esc_url( $retry_url ); ?>">&nbsp<?php esc_html_e( 'choose "Approve" on the permission screen', 'peachpay-for-woocommerce' ); ?></a>.</p>
		<?php
}

/**
 * Renders the stripe signup or stripe connect details.
 */
function peachpay_field_stripe_cb() {
	?>
	<div>
	<?php
	if ( get_option( 'peachpay_connected_stripe_account' ) && get_option( 'peachpay_connected_stripe_account' )['id'] && '' !== trim( get_option( 'peachpay_connected_stripe_account' )['id'], ' ' ) ) {
		?>
		<p>
			<span class="dashicons dashicons-yes-alt"></span> <?php esc_html_e( "You've successfully connected the Stripe account", 'peachpay-for-woocommerce' ); ?>&nbsp<strong><?php echo esc_html( get_option( 'peachpay_connected_stripe_account' )['id'] ); ?></strong>
		</p>
		<br>
		<a class="button button-unlink-stripe" href="?page=peachpay&tab=payment&unlink_stripe&merchant_store=<?php echo esc_url( get_site_url() ); ?>" ><?php esc_html_e( 'Unlink Stripe', 'peachpay-for-woocommerce' ); ?></a>
		<?php
	} else {
		?>
		<a
			href="<?php echo esc_url( peachpay_stripe_signup_url( get_site_url(), get_home_url() ) ); ?>"
			class="pp-stripe-connect"
			target="_blank"
		>
			<span><?php esc_html_e( 'Connect with', 'peachpay-for-woocommerce' ); ?></span>
		</a>
		<?php
	}
	?>
	</div>
	<?php
}

/**
 * Renders the more stripe info.
 */
function peachpay_field_stripe_more_info_cb() {
	?>
	<div class="pp-stripe-container pp-card-bottom" style="display: block; text-align: center; min-height: 35px;">
		<p style="width: 100%;">
			<?php esc_html_e( 'Learn more about', 'peachpay-for-woocommerce' ); ?>
			<a href="https://stripe.com/payments/payment-methods-guide" target="_blank"><?php esc_html_e( 'payment methods', 'peachpay-for-woocommerce' ); ?></a>
			<?php esc_html_e( 'powered by Stripe and any associated', 'peachpay-for-woocommerce' ); ?>
			<a href="https://stripe.com/pricing/local-payment-methods" target="_blank">
				<?php esc_html_e( 'fees', 'peachpay-for-woocommerce' ); ?>
			</a>
		</p>
	</div>
	<style>
			.pp-stripe-container{
				background-color: white;
				border-top: 1pt solid rgba(0,0,0,0.1);
				min-height: 100px;
				padding: 1rem 0;
				margin: 0;
			}
			.pp-stripe-main{
				display: flex;
				flex-direction: row;
				align-items: center;
			}
			.pp-stripe-badge img{
				width: 4.5rem;
				padding: 0 1rem;
				display: inline-block;
			}
			.pp-stripe-mini-badge{
				display: flex;
				flex-direction: column;
				align-content: center;
				padding: 0 1rem 0 0;
			}
			.pp-stripe-mini-badge div {
				display: flex;
				flex-direction: row;
				align-content: center;
			}
			.pp-stripe-mini-badge img{
				display: inline-block;
				width: 3rem;
				padding: 0.2rem 0.2rem;
			}

			.pp-stripe-body{
				display: flex;
				flex-direction: column;
				margin: 0;
				padding: 0;
				flex-grow: 1;
			}

			.pp-stripe-body h3{
				margin: 0 0 5px;
				padding: 0;
				font-size: 1rem;
			}

			.pp-stripe-fees{
				white-space: nowrap;
				border-radius: 0.8rem;
				border: solid rgba(0,0,0) 1px;
				padding: 5px 0.5rem;
				margin: 0 0 0 1rem;
				font-size: smaller;
				opacity: 0.6;
				transition: opacity 0.2s;
			}

			.pp-stripe-fees a{
				display: inline-block;
				width: 125px;
				text-align: center;
				color: black;
				text-decoration: none;
			}
			.pp-stripe-fees:hover{
				opacity: 1;
			}

			.pp-mobile-vertical{
				display: flex;
				flex-direction: row;
				align-items: center;
			}

			.pp-needs-action{
				display: inline-block;
				background-color: rgba(0,0,0,0.1);
				padding: 5px 1rem;
				margin: 1rem 0 0;
				border-radius: 0.5rem;
				width: calc(100% - 2rem);
			}

			.pp-fill{
				flex-grow: 100;
			}
			tr.pp-condense-row, tr.pp-condense-row td{
				margin: 0;
				padding: 0;
			}

			.pp-card-top{
				overflow: hidden;
			}

			.pp-card-bottom{
				overflow: hidden;
			}

			@media screen and (max-width: 782px) {
				.pp-mobile-vertical{
					flex-direction: column;
				}

				.pp-stripe-fees{
					margin: 1rem 0 0;
				}
			}
		</style>
		<script>
			function peachpay_markActionNeeded(stripeMethod){
				const $input = document.querySelector(`input[name="peachpay_payment_options[${stripeMethod}]"]`);
				if(!$input){
					return;
				}

				const $container = $input.closest(".pp-stripe-container");
				if(!$container){
					return;
				}

				$container.insertAdjacentHTML( "beforeend", `
<span class="pp-needs-action">
<b>
	<?php esc_html_e( 'Action required:', 'peachpay-for-woocommerce' ); ?>
</b>
</br>
	<?php esc_html_e( 'This payment method must be activated in your', 'peachpay-for-woocommerce' ); ?>
	<a target="_blank" href="https://dashboard.stripe.com/test/settings/payment_methods">Stripe <?php esc_html_e( 'dashboard', 'peachpay-for-woocommerce' ); ?></a>
	<?php esc_html_e( 'settings to be used in the PeachPay checkout.', 'peachpay-for-woocommerce' ); ?>
</span>`);
			}

			function peachpay_markApplePayConfigError(){
				const $container = document.querySelector("[data-stripe-applepay-configured]");
				if(!$container){
					return;
				}

				const $input = $container.querySelector('input[type="checkbox"]');

				if(!$input || !$input.checked){
					return;
				}

				const status = $container.dataset.stripeApplepayConfigured;
				if(status){
					return
				}


				$container.insertAdjacentHTML( "beforeend", `
<span class="pp-needs-action">
<b>
	<?php esc_html_e( 'Setup failure:', 'peachpay-for-woocommerce' ); ?>
</b>
</br>
	<?php
	if ( peachpay_stripe_supported_applepay_url() ) {
		esc_html_e( 'Apple Pay domain registration failed. Apple Pay will not be shown to customers but Google Pay will continue to work.', 'peachpay-for-woocommerce' );
		?>
		<div style="margin: 5px; width: 100%; text-align: center;">
			<a href="<?php echo esc_url( add_query_arg( 'register_domain', 'true' ) ); ?>" class="button-secondary pp-reset-button">
			<?php esc_html_e( 'Retry Domain Registration', 'peachpay-for-woocommerce' ); ?>
			</a>
		</div>
		<?php
	} else {
		esc_html_e( 'Apple Pay is not supported on localhost sites. Google Pay will continue to work normally.', 'peachpay-for-woocommerce' );
	}
	?>
</span>`);
			}

			const $targets = document.querySelectorAll("[data-stripe-payment]");
			$targets.forEach(($el) => {
				const type = $el.dataset.stripePayment;
				const capabilityType = $el.dataset.stripeCapabilityType;
				const capability = $el.dataset.stripeCapability;

				if(capability !== "active"){
					peachpay_markActionNeeded(type);
				}
			});

			peachpay_markApplePayConfigError();
		</script>
	<?php
}

/**
 * Echos HTML escaped information if the array key exist.
 *
 * @param string $key The key to check if exists.
 * @param array  $array The array to check if the key exists in.
 */
function peachpay_echo_exist( $key, $array ) {
	if ( array_key_exists( $key, $array ) ) {
		echo esc_html( $array[ $key ] );
	}
}

/**
 * Template function for rendering stripe payment methods.
 *
 * @param string       $key The payment type key.
 * @param string       $name The name of the payment method.
 * @param string|array $image The URL of the image for the payment method.
 * @param string       $description The description for the payment method.
 * @param string       $fees The fees short string details.
 * @param array        $options Any extra information needed for rendering.
 */
function peachpay_stripe_payment_option( $key, $name, $image, $description, $fees, $options = array() ) {
	$has_capability = ( peachpay_is_test_mode() && peachpay_get_settings_option( 'peachpay_payment_options', 'enable_stripe' ) ) || ( peachpay_stripe_enabled() && 'active' === $options['data-stripe-capability'] );

	?>
	<div class="pp-stripe-container <?php peachpay_echo_exist( 'class', $options ); ?>"
		data-stripe-payment="<?php echo esc_html( $key ); ?>"
		data-stripe-capability-type="<?php echo esc_html( $options['data-stripe-capability-type'] ); ?>"
		data-stripe-capability="<?php echo esc_html( $options['data-stripe-capability'] ); ?>"
		<?php if ( array_key_exists( 'data-stripe-applepay-configured', $options ) ) { ?>
			data-stripe-applepay-configured="<?php echo esc_html( $options['data-stripe-applepay-configured'] ); ?>"
		<?php } ?>
	>

		<?php if ( ! ( array_key_exists( 'no-input', $options ) && $options['no-input'] ) ) { ?>
		<label>
		<?php } ?>

		<div class="pp-stripe-main">
			<div>
				<?php if ( ! ( array_key_exists( 'no-input', $options ) && $options['no-input'] ) ) { ?>
					<input
						id="peachpay_stripe_<?php echo esc_html( $key ); ?>"
						name="peachpay_payment_options[<?php echo esc_html( $key ); ?>]"
						type="checkbox"
						value="1"
						<?php
						checked( 1, peachpay_get_settings_option( 'peachpay_payment_options', $key, false ) ? 1 : 0, true );
						disabled( false, $has_capability, true );
						?>
					>
					<?php
				}
				?>
			</div>
			<?php if ( ! is_array( $image ) ) { ?>
			<div class="pp-stripe-badge">
				<img src="<?php echo esc_url( $image ); ?>">
			</div>
			<?php } else { ?>
			<div class="pp-stripe-mini-badge">
				<?php
				$length = count( $image ) - 1;
				for ( $index = 0; $index < $length; $index += 2 ) {
					?>
					<div>
						<img src="<?php echo esc_url( $image[ $index ] ); ?>">
						<img src="<?php echo esc_url( $image[ $index + 1 ] ); ?>">
					</div>
				<?php } ?>
			</div>
			<?php } ?>
			<div class="pp-mobile-vertical pp-fill">
				<div class="pp-stripe-body">
					<div>
						<h3>
						<?php echo esc_html( $name ); ?>
						</h3>
					</div>
					<div>
						<p>
						<?php echo esc_html( $description ); ?>
						</p>
					</div>
				</div>
				<div class="pp-stripe-fees">
					<a href="<?php echo esc_url( $fees ); ?>" target="_blank">
						<?php echo esc_html_e( 'View Stripe fees', 'peachpay-for-woocommerce' ); ?>
					</a>
				</div>
			</div>
		</div>

		<?php do_action( 'peachpay_action_stripe_' . $key . '_extra_settings' ); ?>

		<?php if ( ! ( array_key_exists( 'no-input', $options ) && $options['no-input'] ) ) { ?>
		</label>
		<?php } ?>
	</div>
	<?php
}

/**
 * Renders the PayPal signup link or connected PayPal account info.
 */
function peachpay_field_paypal_cb() {
	?>
	<div>
	<?php
	$paypal_merchant_id = '';
	if ( get_option( 'peachpay_paypal_signup' ) ) {
		$paypal_merchant_id = peachpay_get_paypal_merchant_id();
	}

	if ( get_option( 'peachpay_paypal_signup' ) ) {
		?>
		<p>
			<span class="dashicons dashicons-yes-alt"></span> <?php esc_html_e( "You've successfully connected your PayPal account with Merchant ID", 'peachpay-for-woocommerce' ); ?>&nbsp<b><?php echo esc_html( $paypal_merchant_id ); ?></b>
		</p>
		<br>
		<a class="button button-unlink-paypal" href="?page=peachpay&tab=payment&unlink_paypal" ><?php esc_html_e( 'Unlink PayPal', 'peachpay-for-woocommerce' ); ?></a>
		<?php
	} else {
		?>
		<a href="<?php echo esc_url( peachpay_paypal_signup_url() ); ?>" title="PayPal" target="_blank">
			<img
				src="https://www.paypalobjects.com/webstatic/mktg/Logo/pp-logo-150px.png"
				alt="PayPal Logo"
				width="150"
				height="38">
			<div style="margin-top: 0.5rem;">
		<?php esc_html_e( 'Connect your PayPal business account to start accepting payments through PayPal', 'peachpay-for-woocommerce' ); ?>
			</div>
		</a>
		<?php
	}

	?>
	</div>
	<?php
}

/**
 * Display an alert if the merchant has connected at least one payment
 * method but has none selected to show in the checkout window.
 */
function peachpay_connected_payments_check() {
	if ( peachpay_is_test_mode() ) {
		return;
	}

	if ( ! get_option( 'peachpay_connected_stripe_account' ) && ! get_option( 'peachpay_paypal_signup' ) ) {
		// No payment methods have been connected, so we don't need to show the alert.
		return;
	}

	if ( peachpay_paypal_enabled() || peachpay_stripe_enabled() ) {
		// At least one of the connected payment methods is enabled.
		return;
	}

	// At this point, there must be at least one payment method connected but none of them are enabled.
	add_filter( 'admin_notices', 'peachpay_display_payment_method_notice' );
}

/**
 * Filter function for displaying admin notices.
 */
function peachpay_display_payment_method_notice() {
	?>
	<div class="error notice">
		<p>
		<?php
		esc_html_e(
			'You have disabled all PeachPay payment methods. The PeachPay checkout window will appear, but customers will have no way to pay. Please ',
			'peachpay-for-woocommerce'
		);
			$payment_settings = admin_url() . 'admin.php?page=peachpay&tab=payment';
		?>

			<a href="<?php echo esc_url_raw( $payment_settings ); ?>">

			<?php
			esc_html_e(
				'enable at least one payment method',
				'peachpay-for-woocommerce'
			);

			echo '</a>.'
			?>
		</p>
	</div>
		<?php
}
